#include "GLObject.h"

#ifndef Field_h
#define Field_h

// 床や空，柵
class Field : public GLObject {
private:
    float fieldSize; // フィールドの大きさ
    float split; // 分割数
    float tileSize; // 分割後のタイルの大きさ
    float tileHeight; // 床のデコボコの高さ
    
public:
    Field(){
        reset();
    }
    
    void reset(){
        GLObject::reset();
        
        fieldSize = 32;
        split = 16;
        tileSize = fieldSize/split;
        tileHeight = 1.0;
    }
 
    void display(){
        GLObject::display();
        
        glMaterialf(GL_FRONT, GL_AMBIENT, 0.1);
        glMaterialf(GL_FRONT, GL_SPECULAR, 0.1);
        glMaterialf(GL_FRONT, GL_SHININESS, 0.0);
        
        // 床
        for (int z=0; z<split; z++) {
            for (int x=0; x<split; x++) {
                float baseX = -fieldSize/2 + x*tileSize;
                float baseZ = -fieldSize/2 + z*tileSize;
                
                if ((x+z)%2 == 0) {
                    glMaterialfv(GL_FRONT, GL_DIFFUSE, GLColor::GREEN);
                }else{
                    glMaterialfv(GL_FRONT, GL_DIFFUSE, GLColor::GREEN2);
                }
                
                glBegin(GL_QUADS);
                glNormal3d(0, 1, 0);
                glVertex3d(baseX , (z%2==1)&&(x%2==1)?tileHeight:0, baseZ);
                glVertex3d(baseX+tileSize, (z%2==1)&&(x%2==0)?tileHeight:0, baseZ);
                glVertex3d(baseX+tileSize, (z%2==0)&&(x%2==0)?tileHeight:0, baseZ+tileSize);
                glVertex3d(baseX, (z%2==0)&&(x%2==1)?tileHeight:0, baseZ+tileSize);
                glEnd();
                
            }
        }
        
        // 壁奥
        // enemy_levelが上がるにつれて色を灰色から赤色に変えていく
        GLfloat gradred[] = {0.4-enemy_level*0.05 + enemy_level*0.1, 0.4-enemy_level*0.05, 0.4-enemy_level*0.05, 0};
        glMaterialfv(GL_FRONT, GL_DIFFUSE, gradred);
        
        for(int i=-16; i<=16; i++){
            glPushMatrix();
            glTranslatef(i, 0.5, -16);
            glRotatef(-90, 1, 0, 0);
            glutSolidCone(0.5, 2, 5, 5);
            glPopMatrix();
        }
        // 壁手前
        for(int i=-16; i<=16; i++){
            glPushMatrix();
            glTranslatef(i, 0.5, 16);
            glRotatef(-90, 1, 0, 0);
            glutSolidCone(0.5, 2, 5, 5);
            glPopMatrix();
        }
        // 壁右
        for(int i=-16; i<=16; i++){
            glPushMatrix();
            glTranslatef(16, 0.5, i);
            glRotatef(-90, 1, 0, 0);
            glutSolidCone(0.5, 2, 5, 5);
            glPopMatrix();
        }
        // 壁左
        for(int i=-16; i<=16; i++){
            glPushMatrix();
            glTranslatef(-16, 0.5, i);
            glRotatef(-90, 1, 0, 0);
            glutSolidCone(0.5, 2, 5, 5);
            glPopMatrix();
        }
        
        // 下の空
        glMaterialfv(GL_FRONT, GL_DIFFUSE, GLColor::SKYBLUE);
        glBegin(GL_QUADS);
        
        // 床
        glNormal3d(0, 1, 0);
        glVertex3d(-32.0, -1.0, -32.0);
        glVertex3d(32.0, -1.0, -32.0);
        glVertex3d(32.0, -1.0, 32.0);
        glVertex3d(-32.0, -1.0, 32.0);
        glEnd();
    }
};


#endif
