#include "GLObject.h"
#include "Ball.h"
#include "GLColor.h"

#ifndef Tank_h
#define Tank_h

class Tank : public GLObject {
protected:
	// 画面外から出ないようにする
	void validatePosition(){
		if(x > 15.5) x = 15;
        else if(x < -15.5) x = -15;
        if(z > 15.5) z = 15;
        else if(z < -15.5) z = -15;
	}

	bool isInField(){
		if(x > 15) return false;
        else if(x < -15) return false;
        if(z > 15) return false;
        else if(z < -15) return false;
		return true;
	}

	// 玉を発射するための設定
    void setShoot(float delta){
        lastshoot = global_time;
        ball[tid].x = x;
        ball[tid].z = z;
        ball[tid].rotate = rotate;
        ball[tid].type = type;
        ball[tid].expTimer = 0;
        ball[tid].y = 0.9;
        float fR = 360.0 - (ball[tid].rotate + 90.0) + delta;
        ball[tid].rotate = rotate-delta;
        ball[tid].vec_x = 0.5*cos(fR * PI / 180.0);
        ball[tid].vec_z = 0.5*sin(fR * PI / 180.0);
        ball[tid].isExist = true;
        if(tid < MAX_BALL-1) tid++;
        else tid = 0;
    }

	// レベルに応じて発射角度を変更
    void multiShoot(){
		if(!isExist) return;

        setShoot(0);
        if(enemy_level >= 3){
            setShoot(180);
        }
        if(enemy_level >= 6){
            setShoot(60);
            setShoot(-60);
        }
        if(enemy_level >= 9){
            setShoot(90);
            setShoot(-90);
        }
    }

	void displayTank(){
		// type に依存
        glPushMatrix();
        
        if(type == PLAYER){
            glTranslatef(x, y+0.5, z );
            glRotatef(rotate+90, 0.0, 1.0, 0.0);
        }else{
            glTranslatef(x, y+0.5, z );
            glRotatef(rotate+90, 0.0, 1.0, 0.0);
        }
        
        glMaterialf(GL_FRONT, GL_AMBIENT, 0.1);
        glMaterialf(GL_FRONT, GL_SPECULAR, 0.1);
        glMaterialf(GL_FRONT, GL_SHININESS, 0);
        
        glMaterialfv(GL_FRONT, GL_DIFFUSE, type == PLAYER ? GLColor::RED : GLColor::BLUE);	
        glPushMatrix();
        glTranslatef(0, 0.5, 0);
        glutSolidCube(1);
        glPopMatrix();
        
        // タイヤシャフト右
        if (life > maxLife*2/3) {
            glPushMatrix();
            glTranslatef(0, 0.5, 0.5);
            glutSolidCone(0.5, 1.5, 10, 10);
            glPopMatrix();
        }
        
        // タイヤシャフト左
        if (life > maxLife*1/3) {
            glPushMatrix();
            glTranslatef(0, 0.5, -0.5);
            glRotatef(180, 0, 1, 0);
            glutSolidCone(0.5, 1.5, 10, 10);
            glPopMatrix();
        }
        
        // フロントの針
        glMaterialf(GL_FRONT, GL_SHININESS, 60);
        glMaterialf(GL_FRONT, GL_SPECULAR, 1.0);
        glMaterialfv(GL_FRONT, GL_DIFFUSE, GLColor::WHITE);
        glPushMatrix();
        glTranslatef(0.5, 0.5, 0);
        glRotatef(90, 0, 1, 0);
        glutSolidCone(0.5, 0.5+2*life/maxLife, 10, 10);
        glPopMatrix();
        
        // タイヤ
        glMaterialf(GL_FRONT, GL_SHININESS, 0);
        glMaterialf(GL_FRONT, GL_SPECULAR, 0.1);
        glMaterialfv(GL_FRONT, GL_DIFFUSE, GLColor::BLACK);
        
        if (life > maxLife*2/3) {
            glPushMatrix();
            glTranslatef(0, 0.5, 1);
            glRotatef(type == PLAYER ? whrightrot : whrightrot, 0, 0, 1);
            glutSolidTorus(0.3, 1.0, 5, 5);
            glPopMatrix();
        }
        
        if (life > maxLife*1/3) {
            glPushMatrix();
            glTranslatef(0, 0.5, -1);
            glRotatef(type == PLAYER ? whrightrot : whrightrot, 0, 0, 1);
            glutSolidTorus(0.3, 1.0, 5, 5);
            glPopMatrix();
        }
        
        glPopMatrix();
	}

	void displayExp(float x, float y, float z){
		glPushMatrix();
        glTranslatef(x, y, z);
        //glRotatef(rotate, 0.0, 1.0, 0.0);
        glMaterialfv(GL_FRONT, GL_DIFFUSE, GLColor::RED);
        glRotatef(expTimer*72, 0, 1, 0);
		//glutWireSphere(0.5+0.05*expTimer*expTimer, 10, 10);
		glutSolidSphere(0.5+0.05*expTimer*expTimer, 4, 5);
        glPopMatrix();
	}

public:
    Ball ball[MAX_BALL]; // 弾丸
    int tid; // 玉id
    
    float whrightrot, whleftrot; // タイヤの回転角度
    int maxLife; // 開始時の寿命
	int life; // 寿命
    bool attackable; // true なら攻撃可能、falseなら攻撃不可
    unsigned long long lastshoot; // 最後に玉を撃った時刻
    int type;
	int expTimer; // 爆発アニメーション用のタイマ
    
    // コンストラクタではあえてsuperのコンストラクタを呼ばない（resetの2重呼び出しになってしまう）
    Tank(){
        reset();
    }
    
    void reset(){
        GLObject::reset();
        
        whrightrot = whleftrot = 0;
        isExist = true;
        lastshoot = global_time;
        maxLife = life = pow((enemy_level+1), 2.0);
        attackable = true;
		expTimer = -1;
        
        tid = 0;
        for (int i=0; i<MAX_BALL; i++) {
            ball[i].reset();
        }
    }

	void setStatus(Tank &tank){
		GLObject::setStatus(tank);
		/*
		x = tank.x;
		y = tank.y;
		z = tank.z;
		rotate = tank.rotate;
		*/
		life = tank.life;
		maxLife = tank.maxLife;
		expTimer = tank.expTimer;
		lastshoot = tank.lastshoot;

		tid = tank.tid;
        for (int i=0; i<tid; i++) {
            ball[i].setStatus(tank.ball[i]);
			tank.ball[i].isExist = false;
        }
	}
    
    void move(float delta){
        GLObject::move(delta);
        
        if(delta > 0){
            whrightrot -= 10;
            whleftrot -= 10;
        }else{
            whrightrot += 10;
            whleftrot += 10;
        }
    }
    
    void proc(Tank &tank){
        GLObject::proc();
		if(!isExist) return;
		
        // 弾丸の処理
        for (int i=0; i<MAX_BALL; i++) {
            if(ball[i].isExist){
                // 範囲外なら
                if(ball[i].x > 15.5 || ball[i].x < -15.5 || ball[i].z > 15.5 || ball[i].z < -15.5){
                    ball[i].isExist = false;
                    ball[i].expTimer = 0;
                    ball[i].last_type = 0;
                }else{
                    ball[i].x += ball[i].vec_x;    
                    ball[i].z += ball[i].vec_z;
                }
            }
        }
    }
    
    void display(){
        GLObject::display();
		if(!isExist) return;

		if(expTimer == -1){
            displayTank();
			// 玉
			for(int i=0; i<MAX_BALL; i++){
				ball[i].display();
			}
        }else if(expTimer < 5){
			displayExp(x+1, y, z+1);
			displayExp(x-1, y, z-1);
			//displayExp(x+1, y, z-0.5);
			expTimer++;
		}else if(expTimer < 10){
			displayExp(x-1, y, z+1);
			displayExp(x+1, y, z-1);
			expTimer++;
        }else if(expTimer == 10){
			isExist = false;
		}
        
    }
    
    // 相手のタンクが自分のタンクの弾丸に触れているか
    // 触れていたら適切に処理
    // このcheckがきっかけになって相手のタンクが破壊された場合trueを返す
    bool check(Tank &t){
		// なかったり，爆発アニメーション中はチェックしない
        if (!isExist || !t.isExist || t.expTimer != -1) {
            return false;
        }
        
        for(int i=0; i<MAX_BALL; i++){
            if(ball[i].isExist){
                // 玉と相手の当たり判定
                // もし玉と相手の距離が指定値以下だったら
                if (ball[i].distance(t) <= 1.0 + life/maxLife) {
                    t.life--;
                    ball[i].isExist = false;
                    ball[i].expTimer = 0;
                    ball[i].last_type = 1;
                    if (t.life <= 0) {
                        //t.isExist = false;
						t.expTimer = 0;
                        return true;
                    }
                }
            }
        }
        
        return false;
    }
    
	void shoot(){
		if (AUTOPLAY) {
			if(ball[tid].isExist == 0 && attackable == 1 && global_time - lastshoot > 1.0){
				multiShoot();
			}
		}else{
			if(ball[tid].isExist == 0 && attackable == 1 && global_time - lastshoot > 10.0-enemy_level){
				multiShoot();
			}
		}
	}
};

#endif



